/*
 * evolution2_sync - A plugin for the opensync framework
 * Copyright (C) 2004-2005  Armin Bauer <armin.bauer@opensync.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 */
 
#include "evolution2_sync.h"

osync_bool evo2_todo_open(evo_environment *env, OSyncError **error)
{
	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, env);
	ESourceRegistry *registry = NULL;
	ESource *source = NULL;
	EClient *client;
	GError *gerror = NULL;
	
	registry = e_source_registry_new_sync (NULL, &gerror);
	if (!registry) {
		osync_error_set(error, OSYNC_ERROR_GENERIC, "Error getting source registry: %s", gerror ? gerror->message : "None");
		osync_trace(TRACE_EXIT_ERROR, "EVO2-SYNC: %s: %s", __func__, osync_error_print(error));
		g_clear_error(&gerror);
		return FALSE;
	}

	if (strcmp(env->tasks_uid, "default")) {
		source = e_source_registry_ref_source (registry, env->tasks_uid);
		if (!source) {
	  		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to find source for tasks");
			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
			g_object_unref (registry);
			return FALSE;
		}
	} else {
		source = e_source_registry_ref_default_task_list (registry);
		if (!source) {
	  		osync_error_set(error, OSYNC_ERROR_GENERIC, "Failed to find default task list: %s", gerror ? gerror->message : "None");
			env->calendar = NULL;
			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
			g_clear_error(&gerror);
			g_object_unref (registry);
			return FALSE;
		}
	}
	
	client = e_cal_client_connect_sync(source, E_CAL_CLIENT_SOURCE_TYPE_TASKS, -1, NULL, &gerror);
	env->tasks = client ? E_CAL_CLIENT (client) : NULL;
	if(!env->tasks) {
		osync_error_set(error, OSYNC_ERROR_GENERIC, "Failed to create new tasks: %s", gerror ? gerror->message : "None");
		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
		g_object_unref (source);
		g_object_unref (registry);
		g_clear_error(&gerror);
		return FALSE;
	}
	
	if (!osync_anchor_compare(env->member, "todo", env->tasks_uid))
		osync_member_set_slow_sync(env->member, "todo", TRUE);
	
	osync_trace(TRACE_EXIT, "%s", __func__);
	g_object_unref (source);
	g_object_unref (registry);
	return TRUE;
}

void evo2_todo_get_changes(OSyncContext *ctx)
{
	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, ctx);
	evo_environment *env = (evo_environment *)osync_context_get_plugin_data(ctx);
	
	GSList *changes = NULL;
	GSList *l = NULL;
	GError *gerror = NULL;
	
	if (osync_member_get_slow_sync(env->member, "todo") == FALSE)
		osync_debug("EVO2-SYNC", 4, "Ignoring no slow_sync for todo, doing slow_sync instead");
	else
		osync_debug("EVO2-SYNC", 4, "slow_sync for todo");

        if (!e_cal_client_get_object_list_as_comps_sync(env->tasks, "(contains? \"any\" \"\")", &changes, NULL, &gerror)) {
		osync_context_send_log(ctx, "Unable to get all todos");
		osync_trace(TRACE_EXIT_ERROR, "%s: Unable to get all todos: %s", __func__, gerror ? gerror->message : "None");
		g_clear_error(&gerror);
		return;
        }

	for (l = changes; l; l = l->next) {
		ECalComponent *comp = E_CAL_COMPONENT (l->data);
		char *data = e_cal_client_get_component_as_string(env->calendar, e_cal_component_get_icalcomponent(comp));
		const char *uid;
		int datasize = strlen(data) + 1;
		uid = e_cal_component_get_uid(comp);
		evo2_report_change(ctx, "todo", "vtodo20", data, datasize, uid, CHANGE_ADDED);
	}

	g_slist_free_full(changes, g_object_unref);

	osync_trace(TRACE_EXIT, "%s", __func__);
}


static osync_bool evo2_todo_modify(OSyncContext *ctx, OSyncChange *change)
{
	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, ctx, change);
	evo_environment *env = (evo_environment *)osync_context_get_plugin_data(ctx);
	
	const char *uid = osync_change_get_uid(change);
	char *data = osync_change_get_data(change);
	ICalComponent *icomp = NULL, *subcomp;
	char *returnuid = NULL;
	GError *gerror = NULL;
	
	switch (osync_change_get_changetype(change)) {
		case CHANGE_DELETED:
			if (!e_cal_client_remove_object_sync(env->tasks, uid, NULL, E_CAL_OBJ_MOD_ALL, E_CAL_OPERATION_FLAG_NONE, NULL, &gerror)) {
				osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to delete todo: %s", gerror ? gerror->message : "None");
				osync_trace(TRACE_EXIT_ERROR, "%s: Unable to delete todo: %s", __func__, gerror ? gerror->message : "None");
				g_clear_error(&gerror);
				return FALSE;
			}
			break;
		case CHANGE_ADDED:
			icomp = i_cal_component_new_from_string(data);
			if (!icomp) {
				osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to convert todo");
				osync_trace(TRACE_EXIT_ERROR, "%s: Unable to convert todo", __func__);
				return FALSE;
			}
			
			subcomp = i_cal_component_get_first_component(icomp, I_CAL_VTODO_COMPONENT);
			if (!subcomp) {
				osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to get todo");
				osync_trace(TRACE_EXIT_ERROR, "%s: Unable to get todo", __func__);
				g_clear_object(&icomp);
				return FALSE;
			}
			
			if (!e_cal_client_create_object_sync(env->tasks, subcomp, E_CAL_OPERATION_FLAG_NONE, &returnuid, NULL, &gerror)) {
				osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to create todo: %s", gerror ? gerror->message : "None");
				osync_trace(TRACE_EXIT_ERROR, "%s: Unable to create todo: %s", __func__, gerror ? gerror->message : "None");
				g_clear_object(&subcomp);
				g_clear_object(&icomp);
				g_clear_error(&gerror);
				return FALSE;
			}
			osync_change_set_uid(change, returnuid);
			g_clear_object(&subcomp);
			g_clear_object(&icomp);
			g_free(returnuid);
			break;
		case CHANGE_MODIFIED:
			icomp = i_cal_component_new_from_string(data);
			if (!icomp) {
				osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to convert todo2");
				osync_trace(TRACE_EXIT_ERROR, "%s: Unable to convert todo2", __func__);
				return FALSE;
			}
			
			subcomp = i_cal_component_get_first_component(icomp, I_CAL_VTODO_COMPONENT);
			if (!subcomp) {
				osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to get todo2");
				osync_trace(TRACE_EXIT_ERROR, "%s: Unable to get todo2", __func__);
				g_clear_object(&icomp);
				return FALSE;
			}
			
			i_cal_component_set_uid(subcomp, uid);
			if (!e_cal_client_modify_object_sync(env->tasks, subcomp, E_CAL_OBJ_MOD_ALL, E_CAL_OPERATION_FLAG_NONE, NULL, &gerror)) {
				osync_trace(TRACE_INTERNAL, "unable to mod todo: %s", gerror ? gerror->message : "None");
				g_clear_error(&gerror);
				if (!e_cal_client_create_object_sync(env->tasks, subcomp, E_CAL_OPERATION_FLAG_NONE, &returnuid, NULL, &gerror)) {
					osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "Unable to create todo: %s", gerror ? gerror->message : "None");
					osync_trace(TRACE_EXIT_ERROR, "%s: Unable to create todo: %s", __func__, gerror ? gerror->message : "None");
					g_clear_object(&subcomp);
					g_clear_object(&icomp);
					g_clear_error(&gerror);
					return FALSE;
				}

				g_free(returnuid);
			}

			g_clear_object(&subcomp);
			g_clear_object(&icomp);
			break;
		default:
			printf("Error\n");
	}
	
	osync_context_report_success(ctx);
	osync_trace(TRACE_EXIT, "%s", __func__);
	return TRUE;
}

void evo2_tasks_setup(OSyncPluginInfo *info)
{
	osync_plugin_accept_objtype(info, "todo");
	osync_plugin_accept_objformat(info, "todo", "vtodo20", NULL);
	osync_plugin_set_commit_objformat(info, "todo", "vtodo20", evo2_todo_modify);
	osync_plugin_set_access_objformat(info, "todo", "vtodo20", evo2_todo_modify);
}
